@@ -1,7 +1,7 @@
-#Copyright ReportLab Europe Ltd. 2000-2012
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/graphics/charts/barcharts.py
-__version__=''' $Id$ '''
+#history https://hg.reportlab.com/hg-public/reportlab/log/tip/src/reportlab/graphics/charts/barcharts.py
+__version__='3.3.0'
 __doc__="""This module defines a variety of Bar Chart components.
 
 The basic flavors are stacked and side-by-side, available in horizontal and
@@ -10,11 +10,13 @@ vertical versions.
 """
 
 import copy, functools
+from ast import literal_eval
 
 from reportlab.lib import colors
 from reportlab.lib.validators import isNumber, isNumberOrNone, isColor, isColorOrNone, isString,\
             isListOfStrings, SequenceOf, isBoolean, isNoneOrShape, isStringOrNone,\
-            NoneOr, isListOfNumbersOrNone, EitherOr, OneOf
+            NoneOr, isListOfNumbersOrNone, EitherOr, OneOf, isInt
+from reportlab.lib.utils import flatten, isStr
 from reportlab.graphics.widgets.markers import uSymbol2Symbol, isSymbol
 from reportlab.lib.formatters import Formatter
 from reportlab.lib.attrmap import AttrMap, AttrMapValue
@@ -25,6 +27,7 @@ from reportlab.graphics.charts.axes import XCategoryAxis, YValueAxis, YCategoryA
 from reportlab.graphics.charts.textlabels import BarChartLabel, NA_Label, NoneOrInstanceOfNA_Label
 from reportlab.graphics.charts.areas import PlotArea
 from reportlab.graphics.charts.legends import _objStr
+from reportlab import xrange, cmp
 
 class BarChartProperties(PropHolder):
     _attrMap = AttrMap(
@@ -59,7 +62,7 @@ class BarChart(PlotArea):
         categoryAxis = AttrMapValue(None, desc='Handle of the category axis.'),
         data = AttrMapValue(None, desc='Data to be plotted, list of (lists of) numbers.'),
         barLabels = AttrMapValue(None, desc='Handle to the list of bar labels.'),
-        barLabelFormat = AttrMapValue(None, desc='Formatting string or function used for bar labels.'),
+        barLabelFormat = AttrMapValue(None, desc='Formatting string or function used for bar labels. Can be a list or list of lists of such.'),
         barLabelCallOut = AttrMapValue(None, desc='Callout function(label)\nlabel._callOutInfo = (self,g,rowNo,colNo,x,y,width,height,x00,y00,x0,y0)',advancedUsage=1),
         barLabelArray = AttrMapValue(None, desc='explicit array of bar label values, must match size of data if present.'),
         reversePlotOrder = AttrMapValue(isBoolean, desc='If true, reverse common category plot order.',advancedUsage=1),
@@ -78,6 +81,7 @@ class BarChart(PlotArea):
     valueAxisGrid=6,
     annotations=7'''),
         categoryNALabel = AttrMapValue(NoneOrInstanceOfNA_Label, desc='Label to use for a group of N/A values.',advancedUsage=1),
+        seriesOrder = AttrMapValue(SequenceOf(SequenceOf(isInt,emptyOK=0,NoneOK=0,lo=1),emptyOK=0,NoneOK=1,lo=1),"dynamic 'mixed' category style case"),
         )
 
     def makeSwatchSample(self, rowNo, x, y, width, height):
@@ -107,6 +111,7 @@ class BarChart(PlotArea):
         else:
             self.categoryAxis = XCategoryAxis()
             self.valueAxis = YValueAxis()
+        self.categoryAxis._attrMap['style'].validate = OneOf('stacked','parallel','parallel_3d','mixed')
 
         PlotArea.__init__(self)
         self.barSpacing = 0
@@ -167,16 +172,49 @@ class BarChart(PlotArea):
         drawing.add(bc)
         return drawing
 
+    def getSeriesOrder(self):
+        bs = getattr(self,'seriesOrder',None)
+        n = len(self.data)
+        if not bs: 
+            R = [(ss,) for ss in xrange(n)]
+        else:
+            unseen = set(range(n))
+            R = []
+            for s in bs:
+                g = [ss for ss in s if 0<=ss<=n]
+                if g:
+                    R.append(tuple(g))
+                    unseen -= set(g)
+            if unseen:
+                R.extend((ss,) for ss in sorted(unseen))
+        self._seriesOrder = R
+
     def _getConfigureData(self):
-        cA = self.categoryAxis
+        cAStyle = self.categoryAxis.style
         data = self.data
-        if cA.style not in ('parallel','parallel_3d'):
-            _data = data
-            data = max(list(map(len,_data)))*[0]
-            for d in _data:
-                for i in range(len(d)):
-                    data[i] = data[i] + (d[i] or 0)
-            data = list(_data) + [data]
+        cc = max(list(map(len,data)))   #category count
+        _data = data
+        if cAStyle not in ('parallel','parallel_3d'):
+            #stacked or mixed
+            data = []
+            def _accumulate(*D):
+                pdata = max((len(d) for d in D))*[0]
+                ndata = pdata[:]
+                for d in D:
+                    for i,v in enumerate(d):
+                        v = v or 0
+                        if v<=-1e-6:
+                            ndata[i] += v
+                        else:
+                            pdata[i] += v
+                data.append(ndata)
+                data.append(pdata)
+            if cAStyle=='stacked':
+                _accumulate(*_data)
+            else:
+                self.getSeriesOrder()
+                for b in self._seriesOrder:
+                    _accumulate(*(_data[j] for j in b))
         self._configureData = data
 
     def _getMinMax(self):
@@ -194,8 +232,12 @@ class BarChart(PlotArea):
 
         # if zero is in chart, put the other axis there, otherwise use low
         crossesAt = vA.scale(0)
-        if crossesAt > org+length or crossesAt<org:
-            crossesAt = org
+        if not vA.forceZero:
+            eps = min(1,1.001*abs(length))
+            end = org+length
+            org, end = min(org,end)-eps, max(org,end)+eps
+            if crossesAt>eps or crossesAt<end:
+                crossesAt = org
         return crossesAt
 
     def _drawFinish(self):
@@ -240,7 +282,7 @@ class BarChart(PlotArea):
                 if k not in Z:
                     raise ValueError('Unknown zIndex variable %r in %r\nallowed variables are\n%s' % (k,Z,'\n'.join(['%s=%r'% (k,Z[k]) for k in sorted(Z.keys())])))
                 try:
-                    v = eval(v,{})  #only constants allowed
+                    v = literal_eval(v) #only constants allowed
                     assert isinstance(v,(float,int))
                 except:
                     raise ValueError('Bad zIndex value %r in clause %r of zIndex\nallowed variables are\n%s' % (v,z,zIndex,'\n'.join(['%s=%r'% (k,Z[k]) for k in sorted(Z.keys())])))
@@ -270,7 +312,6 @@ class BarChart(PlotArea):
         del self._configureData
         return g
 
-
     def calcBarPositions(self):
         """Works out where they go. default vertical.
 
@@ -287,6 +328,7 @@ class BarChart(PlotArea):
         cScale = cA.scale
 
         data = self.data
+
         seriesCount = self._seriesCount = len(data)
         self._rowLength = rowLength = max(list(map(len,data)))
         wG = self.groupSpacing
@@ -297,15 +339,31 @@ class BarChart(PlotArea):
         if clbo=='auto': clbo = flipXY and 'last' or 'first'
         clbo = clbo=='first'
         style = cA.style
-        if style=='parallel':
+        if style=='mixed':
+            ss = self._seriesOrder
+            barsPerGroup = len(ss)
+            wB = barsPerGroup*barWidth
+            wS = (barsPerGroup-1)*barSpacing
+            if barsPerGroup>1:
+                bGapB = barWidth
+                bGapS = barSpacing
+            else:
+                bGapB = bGapS = 0
+            accumNeg = barsPerGroup*rowLength*[0]
+            accumPos = accumNeg[:]
+        elif style in ('parallel','parallel_3d'):
+            barsPerGroup = 1
             wB = seriesCount*barWidth
             wS = (seriesCount-1)*barSpacing
             bGapB = barWidth
             bGapS = barSpacing
         else:
-            accum = rowLength*[0]
+            barsPerGroup = seriesCount
+            accumNeg = rowLength*[0]
+            accumPos = accumNeg[:]
             wB = barWidth
             wS = bGapB = bGapS = 0
+            
         self._groupWidth = groupWidth = wG+wB+wS
         useAbsolute = self.useAbsolute
 
@@ -359,8 +417,6 @@ class BarChart(PlotArea):
             baseLine = vScale(vM)
         self._baseLine = baseLine
 
-        nC = max(list(map(len,data)))
-
         width = barWidth*fB
         offs = 0.5*wG*fG
         bGap = bGapB*fB+bGapS*fS
@@ -376,51 +432,71 @@ class BarChart(PlotArea):
         self._barPositions = []
         aBP = self._barPositions.append
         reversePlotOrder = self.reversePlotOrder
-        for rowNo in range(seriesCount):
-            barRow = []
-            if reversePlotOrder:
-                xVal = seriesCount-1 - rowNo
+
+        def _addBar(colNo,accx):
+            datum = row[colNo]
+            # Ufff...
+            if useAbsolute==7:
+                x = groupWidth*_cscale(colNo) + xVal + org
             else:
-                xVal = rowNo
-            xVal = offs + xVal*bGap
-            row = data[rowNo]
-            for colNo in range(nC):
-                datum = row[colNo]
+                (g, _) = cScale(colNo)
+                x = g + xVal
 
-                # Ufff...
-                if useAbsolute==7:
-                    x = groupWidth*_cscale(colNo) + xVal + org
-                else:
-                    (g, _) = cScale(colNo)
-                    x = g + xVal
-
-                if datum is None:
-                    height = None
-                    y = baseLine
-                else:
-                    if style not in ('parallel','parallel_3d'):
-                        y = vScale(accum[colNo])
-                        if y<baseLine: y = baseLine
-                        accum[colNo] = accum[colNo] + datum
-                        datum = accum[colNo]
+            if datum is None:
+                height = None
+                y = baseLine
+            else:
+                if style not in ('parallel','parallel_3d'):
+                    if datum<=-1e-6:
+                        y = vScale(accumNeg[accx])
+                        if y>baseLine: y = baseLine
+                        accumNeg[accx] += datum
+                        datum = accumNeg[accx]
                     else:
-                        y = baseLine
-                    height = vScale(datum) - y
-                    if -1e-8<height<=1e-8:
-                        height = 1e-8
-                        if datum<-1e-8: height = -1e-8
-                barRow.append(flipXY and (y,x,height,width) or (x,y,width,height))
+                        y = vScale(accumPos[accx])
+                        if y<baseLine: y = baseLine
+                        accumPos[accx] += datum
+                        datum = accumPos[accx]
+                else:
+                    y = baseLine
+                height = vScale(datum) - y
+                if -1e-8<height<=1e-8:
+                    height = 1e-8
+                    if datum<-1e-8: height = -1e-8
+            barRow.append(flipXY and (y,x,height,width) or (x,y,width,height))
 
-            aBP(barRow)
+        if style!='mixed':
+            for rowNo, row in enumerate(data):  #iterate over the separate series
+                barRow = []
+                xVal = barsPerGroup - 1 - rowNo if reversePlotOrder else rowNo
+                xVal = offs + xVal*bGap
+                for colNo in xrange(rowLength): #iterate over categories
+                    _addBar(colNo,colNo)
+                aBP(barRow)
+        else:
+            for sb,sg in enumerate(self._seriesOrder):  #the sub bar nos and series groups
+                style = 'parallel' if len(sg)<=1 else 'stacked'
+                xVal = barsPerGroup - 1 - sb if reversePlotOrder else sb
+                xVal = offs + xVal*bGap
+                for rowNo in sg:    #the individual series
+                    barRow = []
+                    row = data[rowNo]
+                    for colNo in xrange(rowLength): #iterate over categories
+                        _addBar(colNo,colNo*barsPerGroup + sb)
+                    aBP(barRow)
 
     def _getLabelText(self, rowNo, colNo):
         '''return formatted label text'''
         labelFmt = self.barLabelFormat
+        if isinstance(labelFmt,(list,tuple)):
+            labelFmt = labelFmt[rowNo]
+            if isinstance(labelFmt,(list,tuple)):
+                labelFmt = labelFmt[colNo]
         if labelFmt is None:
             labelText = None
         elif labelFmt == 'values':
             labelText = self.barLabelArray[rowNo][colNo]
-        elif type(labelFmt) is str:
+        elif isStr(labelFmt):
             labelText = labelFmt % self.data[rowNo][colNo]
         elif hasattr(labelFmt,'__call__'):
             labelText = labelFmt(self.data[rowNo][colNo])
@@ -439,7 +515,9 @@ class BarChart(PlotArea):
         if anti: value = 0
         a = x + 0.5*width
         nudge = (height>=0 and 1 or -1)*nudge
-        if bt=='hi':
+        if bt=='mid':
+            b = y+height*0.5
+        elif bt=='hi':
             if value>=0:
                 b = y + value + nudge
             else:
@@ -532,7 +610,6 @@ class BarChart(PlotArea):
         g.add(r)
 
     def _makeBars(self,g,lg):
-        lenData = len(self.data)
         bars = self.bars
         br = getattr(self,'barRecord',None)
         BP = self._barPositions
@@ -542,23 +619,19 @@ class BarChart(PlotArea):
         catNNA = {}
         if catNAL:
             CBL = []
-            rowNoL = lenData - 1
+            rowNoL = len(self.data) - 1
             #find all the categories that have at least one value
-            for rowNo in range(lenData):
-                row = BP[rowNo]
-                for colNo in range(len(row)):
-                    x, y, width, height = row[colNo]
+            for rowNo, row in enumerate(BP):
+                for colNo, (x, y, width, height) in enumerate(row):
                     if None not in (width,height):
                         catNNA[colNo] = 1
 
-        for rowNo in range(lenData):
-            row = BP[rowNo]
+        for rowNo, row in enumerate(BP):
             styleCount = len(bars)
             styleIdx = rowNo % styleCount
             rowStyle = bars[styleIdx]
-            for colNo in range(len(row)):
+            for colNo, (x,y,width,height) in enumerate(row):
                 style = (styleIdx,colNo) in bars and bars[(styleIdx,colNo)] or rowStyle
-                x, y, width, height = row[colNo]
                 if None in (width,height):
                     if not catNAL or colNo in catNNA:
                         self._addNABarLabel(lg,rowNo,colNo,x,y,width,height)
@@ -678,8 +751,8 @@ class BarChart(PlotArea):
             lo = self.x
             hi = lo + self.width
             end = self.y+self.height
-            for i in range(lenData):
-                for x, y, w, h in BP[i]:
+            for bp in BP:
+                for x, y, w, h in bp:
                     v = x+w
                     z = y+h
                     aC((min(y,z),max(y,z), min(x,v) - lo, hi - max(x,v)))
@@ -687,8 +760,8 @@ class BarChart(PlotArea):
             lo = self.y
             hi = lo + self.height
             end = self.x+self.width
-            for i in range(lenData):
-                for x, y, w, h in BP[i]:
+            for bp in BP:
+                for x, y, w, h in bp:
                     v = y+h
                     z = x+w
                     aC((min(x,z), max(x,z), min(y,v) - lo, hi - max(y,v)))
@@ -737,15 +810,12 @@ class BarChart(PlotArea):
         cA.configure(self._configureData)
         self.calcBarPositions()
 
-        lenData = len(self.data)
         bars = self.bars
         R = [].append
         BP = self._barPositions
-        for rowNo in range(lenData):
-            row = BP[rowNo]
+        for rowNo, row in enumerate(BP):
             C = [].append
-            for colNo in range(len(row)):
-                x, y, width, height = row[colNo]
+            for colNox, (y, width, height) in enumerate(row):
                 if None in (width,height):
                     na = self.naLabel
                     if na and na.text:
@@ -893,9 +963,14 @@ class BarChart3D(BarChart):
         g = Group()
         theta_x = self.theta_x
         theta_y = self.theta_y
-        if self.categoryAxis.style == 'stacked':
-          fg_value=fg.value().reverse()
-        for t in fg.value():
+        fg_value = fg.value()
+        cAStyle = self.categoryAxis.style
+        if cAStyle=='stacked':
+            fg_value=fg_value.reverse()
+        elif cAStyle=='mixed':
+            fg_value = [_[1] for _ in sorted((((t[1],t[2],t[3],t[4]),t) for t in fg_value))]
+
+        for t in fg_value:
             if t[0]==0:
                 z0,z1,x,y,width,height,rowNo,style = t[1:]
                 dz = z1 - z0
@@ -903,8 +978,8 @@ class BarChart3D(BarChart):
                             fillColor=style.fillColor, fillColorShaded=None,
                             strokeColor=style.strokeColor, strokeWidth=style.strokeWidth,
                             shading=0.45)
-        for t in fg.value():
-            if t[0]==1:
+        for t in fg_value:
+            if t==1:
                 z0,z1,x,y,width,height,rowNo,colNo = t[1:]
                 BarChart._addBarLabel(self,g,rowNo,colNo,x,y,width,height)
         return g
