@@ -1,6 +1,6 @@
 #Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history https://bitbucket.org/rptlab/reportlab/history-node/tip/src/reportlab/graphics/shapes.py
+#history https://hg.reportlab.com/hg-public/reportlab/log/tip/src/reportlab/graphics/shapes.py
 
 __version__='3.3.0'
 __doc__='''Core of the graphics library - defines Drawing and Shapes'''
@@ -695,7 +695,8 @@ class Drawing(Group, Flowable):
         draw itself in a story.  It is specific to PDF and should not
         be used directly."""
         from reportlab.graphics import renderPDF
-        renderPDF.draw(self, self.canv, 0, 0, showBoundary=showBoundary)
+        renderPDF.draw(self, self.canv, 0, 0,
+                showBoundary=showBoundary if showBoundary is not _unset_ else getattr(self,'_showBoundary',_unset_))
 
     def wrap(self, availWidth, availHeight):
         width = self.width
