@@ -1,24 +1,34 @@
-#Copyright ReportLab Europe Ltd. 2000-2012
+#Copyright ReportLab Europe Ltd. 2000-2019
 #see license.txt for license details
 # $URI:$
-__version__=''' $Id$ '''
+__version__='3.5.34'
 __doc__='''Gazillions of miscellaneous internal utility functions'''
 
-import os, sys, imp, time, types
+import os, sys, time, types, datetime, ast
+from functools import reduce as functools_reduce
+literal_eval = ast.literal_eval
 from base64 import decodestring as base64_decodestring, encodestring as base64_encodestring
-try:
-    from cPickle import dumps as pickle_dumps, loads as pickle_loads, dump as pickle_dump, load as pickle_load
-except ImportError:
-    from pickle import dumps as pickle_dumps, loads as pickle_loads, dump as pickle_dump, load as pickle_load
 from reportlab import isPy3
 from reportlab.lib.logger import warnOnce
 from reportlab.lib.rltempfile import get_rl_tempfile, get_rl_tempdir, _rl_getuid
+from . rl_safe_eval import rl_safe_exec, rl_safe_eval, safer_globals
+
+try:
+    import cPickle as pickle
+except ImportError:
+    import pickle
 
 try:
     from hashlib import md5
 except ImportError:
     import md5
 
+try:
+    import platform
+    isPyPy = platform.python_implementation()=='PyPy'
+except:
+    isPyPy = False
+
 def isFunction(v):
     return type(v) == type(isFunction)
 
@@ -114,10 +124,10 @@ if isPy3:
             return str(x).encode(enc)
 
     def encode_label(args):
-        return base64_encodestring(pickle_dumps(args)).strip().decode('latin1')
+        return base64_encodestring(pickle.dumps(args)).strip().decode('latin1')
 
     def decode_label(label):
-        return pickle_loads(base64_decodestring(label.encode('latin1')))
+        return pickle.loads(base64_decodestring(label.encode('latin1')))
 
     def rawUnicode(s):
         '''converts first 256 unicodes 1-1'''
@@ -211,10 +221,10 @@ else:
     from string import letters as ascii_letters, uppercase as ascii_uppercase, lowercase as ascii_lowercase
 
     def encode_label(args):
-        return base64_encodestring(pickle_dumps(args)).strip()
+        return base64_encodestring(pickle.dumps(args)).strip()
 
     def decode_label(label):
-        return pickle_loads(base64_decodestring(label))
+        return pickle.loads(base64_decodestring(label))
 
     def rawUnicode(s):
         '''converts first 256 unicodes 1-1'''
@@ -225,6 +235,7 @@ else:
         return s.encode('latin1') if isinstance(s,unicode) else s
 
     def rl_exec(obj, G=None, L=None):
+        '''this is unsafe'''
         if G is None:
             frame = sys._getframe(1)
             G = frame.f_globals
@@ -233,7 +244,7 @@ else:
             del frame
         elif L is None:
             L = G
-        exec("""exec obj in G, L""")
+        exec(obj,G, L)
     rl_exec("""def rl_reraise(t, v, b=None):\n\traise t, v, b\n""")
 
     char2int = ord
@@ -247,7 +258,7 @@ def zipImported(ldr=None):
         if not ldr:
             ldr = sys._getframe(1).f_globals['__loader__']
         from zipimport import zipimporter
-        return ldr if isinstance(ldr,zipimporter) else None
+        return ldr if isinstance(ldr,zipimporter) and len(ldr._files) else None
     except:
         return None
 
@@ -364,15 +375,18 @@ del reportlab
 #Attempt to detect if this copy of reportlab is running in a
 #file system (as opposed to mostly running in a zip or McMillan
 #archive or Jar file).  This is used by test cases, so that
-#we can write test cases that don't get activated in a compiled
+#we can write test cases that don't get activated in frozen form.
 try:
     __file__
 except:
     __file__ = sys.argv[0]
 import glob, fnmatch
 try:
-    _isFSD = not __loader__
-    _archive = os.path.normcase(os.path.normpath(__loader__.archive))
+    __rl_loader__ = __loader__
+    _isFSD = not __rl_loader__
+    if not zipImported(ldr=__rl_loader__):
+        raise NotImplementedError("can't handle compact distro type %r" % __rl_loader__)
+    _archive = os.path.normcase(os.path.normpath(__rl_loader__.archive))
     _archivepfx = _archive + os.sep
     _archivedir = os.path.dirname(_archive)
     _archivedirpfx = _archivedir + os.sep
@@ -408,11 +422,11 @@ try:
         c, pfn = __startswith_rl(pattern)
         r = glob(pfn)
         if c or r==[]:
-            r += list(map(lambda x,D=_archivepfx,pjoin=pjoin: pjoin(_archivepfx,x),list(filter(lambda x,pfn=pfn,fnmatch=fnmatch: fnmatch(x,pfn),list(__loader__._files.keys())))))
+            r += list(map(lambda x,D=_archivepfx,pjoin=pjoin: pjoin(_archivepfx,x),list(filter(lambda x,pfn=pfn,fnmatch=fnmatch: fnmatch(x,pfn),list(__rl_loader__._files.keys())))))
         return r
 except:
     _isFSD = os.path.isfile(__file__)   #slight risk of wrong path
-    __loader__ = None
+    __rl_loader__ = None
     def _startswith_rl(fn):
         return fn
     def rl_glob(pattern,glob=glob.glob):
@@ -480,22 +494,6 @@ def recursiveImport(modulename, baseDir=None, noCWD=0, debug=0):
     finally:
         sys.path = opath
 
-def recursiveGetAttr(obj, name):
-    "Can call down into e.g. object1.object2[4].attr"
-    return eval(name, obj.__dict__)
-
-def recursiveSetAttr(obj, name, value):
-    "Can call down into e.g. object1.object2[4].attr = value"
-    #get the thing above last.
-    tokens = name.split('.')
-    if len(tokens) == 1:
-        setattr(obj, name, value)
-    else:
-        most = '.'.join(tokens[:-1])
-        last = tokens[-1]
-        parent = recursiveGetAttr(obj, most)
-        setattr(parent, last, value)
-
 def import_zlib():
     try:
         import zlib
@@ -548,9 +546,9 @@ def getArgvDict(**kw):
             elif isinstance(v,int):
                 v = int(av)
             elif isinstance(v,list):
-                v = list(eval(av))
+                v = list(literal_eval(av),{})
             elif isinstance(v,tuple):
-                v = tuple(eval(av))
+                v = tuple(literal_eval(av),{})
             else:
                 raise TypeError("Can't convert string %r to %s" % (av,type(v)))
         return v
@@ -565,7 +563,7 @@ def getArgvDict(**kw):
         handled = 0
         ke = k+'='
         for a in A:
-            if a.find(ke)==0:
+            if a.startswith(ke):
                 av = a[len(ke):]
                 A.remove(a)
                 R[k] = handleValue(v,av,func)
@@ -600,11 +598,11 @@ def open_for_read_by_name(name,mode='b'):
     try:
         return open(name,mode)
     except IOError:
-        if _isFSD or __loader__ is None: raise
-        #we have a __loader__, perhaps the filename starts with
+        if _isFSD or __rl_loader__ is None: raise
+        #we have a __rl_loader__, perhaps the filename starts with
         #the dirname(reportlab.__file__) or is relative
         name = _startswith_rl(name)
-        s = __loader__.get_data(name)
+        s = __rl_loader__.get_data(name)
         if 'b' not in mode and os.linesep!='\n': s = s.replace(os.linesep,'\n')
         return getBytesIO(s)
 
@@ -657,7 +655,7 @@ def open_for_read(name,mode='b', urlopen=urlopen, datareader=datareader):
         return open_for_read_by_name(name,mode)
     except:
         try:
-            return getBytesIO(datareader(name) if name.startswith('data:') else urlopen(name).read())
+            return getBytesIO(datareader(name) if name[:5].lower()=='data:' else urlopen(name).read())
         except:
             raise IOError('Cannot open resource "%s"' % name)
 del urlopen, datareader
@@ -676,34 +674,53 @@ def open_and_readlines(name,mode='t'):
 def rl_isfile(fn,os_path_isfile=os.path.isfile):
     if hasattr(fn,'read'): return True
     if os_path_isfile(fn): return True
-    if _isFSD or __loader__ is None: return False
+    if _isFSD or __rl_loader__ is None: return False
     fn = _startswith_rl(fn)
-    return fn in list(__loader__._files.keys())
+    return fn in list(__rl_loader__._files.keys())
 
 def rl_isdir(pn,os_path_isdir=os.path.isdir,os_path_normpath=os.path.normpath):
     if os_path_isdir(pn): return True
-    if _isFSD or __loader__ is None: return False
+    if _isFSD or __rl_loader__ is None: return False
     pn = _startswith_rl(os_path_normpath(pn))
     if not pn.endswith(os.sep): pn += os.sep
-    return len(list(filter(lambda x,pn=pn: x.startswith(pn),list(__loader__._files.keys()))))>0
+    return len(list(filter(lambda x,pn=pn: x.startswith(pn),list(__rl_loader__._files.keys()))))>0
 
 def rl_listdir(pn,os_path_isdir=os.path.isdir,os_path_normpath=os.path.normpath,os_listdir=os.listdir):
-    if os_path_isdir(pn) or _isFSD or __loader__ is None: return os_listdir(pn)
+    if os_path_isdir(pn) or _isFSD or __rl_loader__ is None: return os_listdir(pn)
     pn = _startswith_rl(os_path_normpath(pn))
     if not pn.endswith(os.sep): pn += os.sep
-    return [x[len(pn):] for x in __loader__._files.keys() if x.startswith(pn)]
+    return [x[len(pn):] for x in __rl_loader__._files.keys() if x.startswith(pn)]
 
 def rl_getmtime(pn,os_path_isfile=os.path.isfile,os_path_normpath=os.path.normpath,os_path_getmtime=os.path.getmtime,time_mktime=time.mktime):
-    if os_path_isfile(pn) or _isFSD or __loader__ is None: return os_path_getmtime(pn)
+    if os_path_isfile(pn) or _isFSD or __rl_loader__ is None: return os_path_getmtime(pn)
     p = _startswith_rl(os_path_normpath(pn))
     try:
-        e = __loader__._files[p]
+        e = __rl_loader__._files[p]
     except KeyError:
         return os_path_getmtime(pn)
     s = e[5]
     d = e[6]
     return time_mktime((((d>>9)&0x7f)+1980,(d>>5)&0xf,d&0x1f,(s>>11)&0x1f,(s>>5)&0x3f,(s&0x1f)<<1,0,0,0))
 
+if isPy3 and sys.version_info[:2]>=(3,4):
+    from importlib import util as importlib_util
+    def __rl_get_module__(name,dir):
+        for ext in ('.py','.pyw','.pyo','.pyc','.pyd'):
+            path = os.path.join(dir,name+ext)
+            if os.path.isfile(path):
+                spec = importlib_util.spec_from_file_location(name,path)
+                return spec.loader.load_module()
+        raise ImportError('no suitable file found')
+else:
+    import imp
+    def __rl_get_module__(name,dir):
+        f, p, desc= imp.find_module(name,[dir])
+        try:
+            return imp.load_module(name,f,p,desc)
+        finally:
+            if f:
+                f.close()
+
 def rl_get_module(name,dir):
     if name in sys.modules:
         om = sys.modules[name]
@@ -711,10 +728,8 @@ def rl_get_module(name,dir):
     else:
         om = None
     try:
-        f = None
         try:
-            f, p, desc= imp.find_module(name,[dir])
-            return imp.load_module(name,f,p,desc)
+            return __rl_get_module__(name,dir)
         except:
             if isCompactDistro():
                 #attempt a load from inside the zip archive
@@ -726,8 +741,6 @@ def rl_get_module(name,dir):
             raise ImportError('%s[%s]' % (name,dir))
     finally:
         if om: sys.modules[name] = om
-        del om
-        if f: f.close()
 
 def _isPILImage(im):
     try:
@@ -857,11 +870,12 @@ class ImageReader(object):
                 else:
                     im = self._image
                     mode = self.mode = im.mode
-                    if mode=='RGBA':
-                        if Image.VERSION.startswith('1.1.7'): im.load()
-                        self._dataA = ImageReader(im.split()[3])
-                        im = im.convert('RGB')
-                        self.mode = 'RGB'
+                    if mode in ('LA','RGBA'):
+                        if getattr(Image,'VERSION','').startswith('1.1.7'): im.load()
+                        self._dataA = ImageReader(im.split()[3 if mode=='RGBA' else 1])
+                        nm = mode[:-1]
+                        im = im.convert(nm)
+                        self.mode = nm
                     elif mode not in ('L','RGB','CMYK'):
                         if im.format=='PNG' and im.mode=='P' and 'transparency' in im.info:
                             im = im.convert('RGBA')
@@ -968,7 +982,7 @@ class DebugMemo:
         md=None
         try:
             import marshal
-            md=marshal.loads(__loader__.get_data('meta_data.mar'))
+            md=marshal.loads(__rl_loader__.get_data('meta_data.mar'))
             project_version=md['project_version']
         except:
             pass
@@ -994,7 +1008,7 @@ class DebugMemo:
                         'version_info': getattr(sys,'version_info','????'),
                         'winver': getattr(sys,'winver','????'),
                         'environment': '\n\t\t\t'.join(['']+['%s=%r' % (k,env[k]) for k in K]),
-                        '__loader__': repr(__loader__),
+                        '__rl_loader__': repr(__rl_loader__),
                         'project_meta_data': md,
                         'project_version': project_version,
                         })
@@ -1044,17 +1058,17 @@ class DebugMemo:
     def _dump(self,f):
         try:
             pos=f.tell()
-            pickle_dump(self.store,f)
+            pickle.dump(self.store,f)
         except:
             S=self.store.copy()
             ff=getBytesIO()
             for k,v in S.items():
                 try:
-                    pickle_dump({k:v},ff)
+                    pickle.dump({k:v},ff)
                 except:
                     S[k] = '<unpicklable object %r>' % v
             f.seek(pos,0)
-            pickle_dump(S,f)
+            pickle.dump(S,f)
 
     def dump(self):
         f = open(self.fn,'wb')
@@ -1069,7 +1083,7 @@ class DebugMemo:
         return f.getvalue()
 
     def _load(self,f):
-        self.store = pickle_load(f)
+        self.store = pickle.load(f)
 
     def load(self):
         f = open(self.fn,'rb')
@@ -1223,7 +1237,7 @@ def simpleSplit(text,fontName,fontSize,maxWidth):
     if maxWidth:
         L = []
         for l in lines:
-            L[-1:-1] = _simpleSplit(l,maxWidth,SW)
+            L.extend(_simpleSplit(l,maxWidth,SW))
         lines = L
     return lines
 
@@ -1436,3 +1450,135 @@ def makeFileName(s):
     if not isUnicode(s):
         s = s.decode('utf8')
     return s
+
+class FixedOffsetTZ(datetime.tzinfo):
+    """Fixed offset in minutes east from UTC."""
+
+    def __init__(self, h, m, name):
+        self.__offset = datetime.timedelta(hours=h, minutes = m)
+        self.__name = name
+
+    def utcoffset(self, dt):
+        return self.__offset
+
+    def tzname(self, dt):
+        return self.__name
+
+    def dst(self, dt):
+        return datetime.timedelta(0)
+
+class TimeStamp(object):
+    def __init__(self,invariant=None):
+        if invariant is None:
+            from reportlab.rl_config import invariant
+        t = os.environ.get('SOURCE_DATE_EPOCH','').strip()
+        if invariant or t:
+            t = int(t) if t else 946684800.0
+            lt = time.gmtime(t)
+            dhh = dmm = 0
+            self.tzname = 'UTC'
+        else:
+            t = time.time()
+            lt = tuple(time.localtime(t))
+            dhh = int(time.timezone / (3600.0))
+            dmm = (time.timezone % 3600) % 60
+            self.tzname = '' 
+        self.t = t
+        self.lt = lt
+        self.YMDhms = tuple(lt)[:6]
+        self.dhh = dhh
+        self.dmm = dmm
+
+    @property
+    def datetime(self):
+        if self.tzname:
+            return datetime.datetime.fromtimestamp(self.t,FixedOffsetTZ(self.dhh,self.dmm,self.tzname))
+        else:
+            return datetime.datetime.now()
+
+    @property
+    def asctime(self):
+        a = time.asctime(self.lt)
+        if not isPy3:
+            a = a.split()
+            a[2] = a[2].lstrip('0')
+            a = ' '.join(a)
+        return a
+
+###############################################################
+#the following code has been (thanks to MIT license)
+#freely adapted from https://github.com/frmdstryr/magicattr
+#the function names are changed to avoid clasing with the actual
+#magicattr names to prevent confusion should others be using that
+#as well as ReportLab
+from functools import reduce as functools_reduce
+
+#: Types of AST nodes that are used
+_raccess_ast_types = (ast.Name, ast.Attribute, ast.Subscript, ast.Call)
+
+def recursiveGetAttr(ob, a):
+    return functools_reduce(_raccess_getattr, _raccess_get_nodes(a), ob)
+
+def recursiveSetAttr(ob, a, val):
+    ob, attr_or_key, is_subscript = raccess_lookup(ob, a)
+    if is_subscript:
+        ob[attr_or_key] = val
+    else:
+        setattr(ob, attr_or_key, val)
+
+def recursiveDelAttr(ob, a):
+    ob, attr_or_key, is_subscript = raccess_lookup(ob, a)
+    if is_subscript:
+        del ob[attr_or_key]
+    else:
+        delattr(ob, attr_or_key)
+
+def raccess_lookup(ob, a):
+    N = tuple(_raccess_get_nodes(a))
+    if len(N) > 1:
+        ob = functools_reduce(_raccess_getattr, N[:-1], ob)
+        n = N[-1]
+    else:
+        n = N[0]
+    if isinstance(n, ast.Attribute):
+        return ob, n.attr, False
+    elif isinstance(n, ast.Subscript):
+        return ob, _raccess_getitem(n.slice.value), True
+    elif isinstance(n, ast.Name):
+        return ob, n.id, False
+    raise NotImplementedError("access by %s is not supported" % n)
+
+def _raccess_get_nodes(a):
+    if not isStr(a):
+        raise TypeError("Attribute name must be a string not %s" % repr(a))
+    if not isNative(a):
+        a = asNative(a)
+    N = ast.parse(a).body
+    if not N or not isinstance(N[0], ast.Expr):
+        raise ValueError("Invalid expression: %s"%a)
+    return reversed([n for n in ast.walk(N[0])
+                     if isinstance(n, _raccess_ast_types)])
+
+def _raccess_getitem(n):
+    # Handle indexes
+    if isinstance(n, ast.Num):
+        return n.n
+    # Handle string keys
+    elif isinstance(n, ast.Str):
+        return n.s
+    # Handle negative indexes
+    elif (isinstance(n, ast.UnaryOp) and isinstance(n.op, ast.USub)
+          and isinstance(n.operand, ast.Num)):
+        return -n.operand.n
+    raise NotImplementedError("subscripting unsupported for node: %s" % n)
+
+def _raccess_getattr(ob, n):
+    if isinstance(n, ast.Attribute):
+        return getattr(ob, n.attr)
+    elif isinstance(n, ast.Subscript):
+        return ob[_raccess_getitem(n.slice.value)]
+    elif isinstance(n, ast.Name):
+        return getattr(ob, n.id)
+    elif isinstance(n, ast.Call):
+        raise ValueError("Function calls are not allowed.")
+    raise NotImplementedError("unsupported node: %s" % n)
