@@ -48,12 +48,24 @@ listWrapOnFakeWidth
 T1SearchPath
 TTFSearchPath
 CMapSearchPath
-baseUnderlineProportion
 decimalSymbol
 errorOnDuplicatePageLabelPage
 autoGenerateMissingTTFName
 allowTTFSubsetting
-spaceShrinkage'''.split())
+spaceShrinkage
+underlineWidth
+underlineOffset
+underlineGap
+strikeWidth
+strikeOffset
+strikeGap
+hyphenationLang
+uriWasteReduce
+embeddedHyphenation
+hyphenationMinWordLength
+reserveTTFNotdef
+documentLang
+encryptionStrength'''.split())
 
 allowTableBoundsErrors =    1 # set to 0 to die on too large elements in tables in debug (recommend 1 for production use)
 shapeChecking =             1
@@ -103,7 +115,21 @@ debug=                      0                       #for debugging code
 rtlSupport=                 0                       #set to 1 to attempt import of RTL assistance eg fribidi etc etc
 listWrapOnFakeWidth=        1                       #set to 0/False to force platypus.flowables._listWrapOn to report correct widths
                                                     #else it reports minimum(required,available) width
-baseUnderlineProportion=    0.0                     #non-zero for doing font size proportional lines in Paragraph.py
+
+underlineWidth=             ''                      #empty to use canvas strokeWidth or a distance or number*<letter>
+                                                    #   num * <letter> make value proportional to a font size
+                                                    #   P paragraph font size
+                                                    #   L line max font size
+                                                    #   f first use font size
+                                                    #   F max fontsize in the tag
+
+underlineOffset=            '-0.125*F'              #fraction of fontSize from baseline to draw underlines at.
+underlineGap=               '1'                     #gap for double/triple underline
+
+strikeWidth=                ''
+strikeOffset=               '0.25*F'                #fraction of fontSize from baseline to draw strike through at.
+strikeGap=                  '1'                     #gap for double/triple strike
+
                                                     #by default typical value 0.05. may be overridden on a parastyle.
 decimalSymbol=              '.'                     #what we use to align floats numerically
 errorOnDuplicatePageLabelPage= 0                    #if True will cause repeated PageLabel page numbers to raise an error.
@@ -116,6 +142,18 @@ allowTTFSubsetting=         []                      #list of font file names tha
                                                     #ReportLab takes no responsibility for the use of this setting.
 
 spaceShrinkage=0.05                                 #allowable space shrinkage to make lines fit
+hyphenationLang=''                                  #if pyphen installed set this to the language of your choice
+                                                    #eg 'en_GB'
+
+uriWasteReduce=0                                    #split URI if we would waste 0.3 of a line or if the URI#
+                                                    #would not fit on the next line; if zero then no splitting
+                                                    #is attempted. suggested value = 0.3
+embeddedHyphenation=0                               #if true attempt hypenation of words with embedded hyphens
+hyphenationMinWordLength=5                          #minimum length of words that can be hyphenated
+reserveTTFNotdef=0                                  #if true force subset element 0 to be zero(.notdef)
+                                                    #helps to fix bug in edge
+documentLang=None                                   #pdf document catalog Lang value xx-xx not ee_xx
+encryptionStrength=40                               #the bits for standard encryption 40, 128 or 256 (AES)
 
 
 # places to look for T1Font information
