@@ -23,13 +23,13 @@ def utf8(code):
     "Convert a given UCS character index into UTF-8"
     return uniChr(code).encode('utf8')
 
-def _simple_subset_generation(fn,npages,alter=0):
+def _simple_subset_generation(fn,npages,alter=0,fonts=('Vera','VeraBI')):
     c = Canvas(outputfile(fn))
     c.setFont('Helvetica', 30)
     c.drawString(100,700, 'Unicode TrueType Font Test %d pages' % npages)
     # Draw a table of Unicode characters
     for p in range(npages):
-        for fontName in ('Vera','VeraBI'):
+        for fontName in fonts:
             c.setFont(fontName, 10)
             for i in range(32):
                 for j in range(32):
@@ -38,6 +38,51 @@ def _simple_subset_generation(fn,npages,alter=0):
         c.showPage()
     c.save()
 
+def show_all_glyphs(fn,fontName='Vera'):
+    c = Canvas(outputfile(fn))
+    c.setFont('Helvetica', 20)
+    c.drawString(72,c._pagesize[1]-30, 'Unicode TrueType Font Test %s' % fontName)
+    from reportlab.pdfbase.pdfmetrics import _fonts
+    from reportlab.lib.utils import uniChr
+    font = _fonts[fontName]
+    doc = c._doc
+    kfunc = font.face.charToGlyph.keys if isPy3 else font.face.charToGlyph.iterkeys
+    for s in kfunc():
+        if s<0x10000:
+            font.splitString(uniChr(s),doc)
+    state = font.state[doc]
+    cn = {}
+    #print('len(assignments)=%d'%  len(state.assignments))
+    nzero = 0
+    ifunc = state.assignments.items if isPy3 else state.assignments.iteritems
+    for code, n in ifunc():
+        if code==0: nzero += 1
+        cn[n] = uniChr(code)
+    if nzero>1: print('%s there were %d zero codes' % (fontName,nzero))
+
+
+    ymin = 10*12
+    y = y0 = c._pagesize[1] - 72
+    for nss,subset in enumerate(state.subsets):
+        if y<ymin:
+            c.showPage()
+            y = y0
+        c.setFont('Helvetica', 10)
+        x = 72
+        c.drawString(x,y,'Subset %d len=%d' % (nss,len(subset)))
+        #print('Subset %d len=%d' % (nss,len(subset)))
+        c.setFont(fontName, 10)
+        for i, code in enumerate(subset):
+            if i%32 == 0:
+                y -= 12
+                x = 72
+            c.drawString(x,y,uniChr(code))
+            x += 13
+        y -= 18
+
+    c.showPage()
+    c.save()
+
 class TTFontsTestCase(unittest.TestCase):
     "Make documents with TrueType fonts"
 
@@ -45,6 +90,13 @@ class TTFontsTestCase(unittest.TestCase):
         "Test PDF generation with TrueType fonts"
         pdfmetrics.registerFont(TTFont("Vera", "Vera.ttf"))
         pdfmetrics.registerFont(TTFont("VeraBI", "VeraBI.ttf"))
+        try:
+            pdfmetrics.registerFont(TTFont("DejaVuSans", "DejaVuSans.ttf"))
+        except:
+            pass
+        else:
+            show_all_glyphs('test_pdfbase_ttfonts_dejavusans.pdf',fontName='DejaVuSans')
+        show_all_glyphs('test_pdfbase_ttfonts_vera.pdf',fontName='Vera')
         _simple_subset_generation('test_pdfbase_ttfonts1.pdf',1)
         _simple_subset_generation('test_pdfbase_ttfonts3.pdf',3)
         _simple_subset_generation('test_pdfbase_ttfonts35.pdf',3,5)
@@ -90,50 +142,50 @@ class TTFontFileTestCase(NearTestCase):
                 self._pos = 0
 
         ttf = FakeTTFontFile(b"\x81\x02\x03\x04" b"\x85\x06" b"ABCD" b"\x7F\xFF" b"\x80\x00" b"\xFF\xFF")
-        self.assertEquals(ttf.read_ulong(), 0x81020304) # big-endian
-        self.assertEquals(ttf._pos, 4)
-        self.assertEquals(ttf.read_ushort(), 0x8506)
-        self.assertEquals(ttf._pos, 6)
-        self.assertEquals(ttf.read_tag(), 'ABCD')
-        self.assertEquals(ttf._pos, 10)
-        self.assertEquals(ttf.read_short(), 0x7FFF)
-        self.assertEquals(ttf.read_short(), -0x8000)
-        self.assertEquals(ttf.read_short(), -1)
+        self.assertEqual(ttf.read_ulong(), 0x81020304) # big-endian
+        self.assertEqual(ttf._pos, 4)
+        self.assertEqual(ttf.read_ushort(), 0x8506)
+        self.assertEqual(ttf._pos, 6)
+        self.assertEqual(ttf.read_tag(), 'ABCD')
+        self.assertEqual(ttf._pos, 10)
+        self.assertEqual(ttf.read_short(), 0x7FFF)
+        self.assertEqual(ttf.read_short(), -0x8000)
+        self.assertEqual(ttf.read_short(), -1)
 
     def testFontFile(self):
         "Tests TTFontFile and TTF parsing code"
         ttf = TTFontFile("Vera.ttf")
-        self.assertEquals(ttf.name, b"BitstreamVeraSans-Roman")
-        self.assertEquals(ttf.flags, FF_SYMBOLIC)
-        self.assertEquals(ttf.italicAngle, 0.0)
+        self.assertEqual(ttf.name, b"BitstreamVeraSans-Roman")
+        self.assertEqual(ttf.flags, FF_SYMBOLIC)
+        self.assertEqual(ttf.italicAngle, 0.0)
         self.assertNear(ttf.ascent,759.765625)
         self.assertNear(ttf.descent,-240.234375)
-        self.assertEquals(ttf.capHeight, 759.765625)
+        self.assertEqual(ttf.capHeight, 759.765625)
         self.assertNear(ttf.bbox, [-183.10546875, -235.83984375, 1287.109375, 928.22265625])
-        self.assertEquals(ttf.stemV, 87)
-        self.assertEquals(ttf.defaultWidth, 600.09765625)
+        self.assertEqual(ttf.stemV, 87)
+        self.assertEqual(ttf.defaultWidth, 600.09765625)
 
     def testAdd32(self):
         "Test add32"
-        self.assertEquals(add32(10, -6), 4)
-        self.assertEquals(add32(6, -10), -4&0xFFFFFFFF)
-        self.assertEquals(add32(0x80000000, -1), 0x7FFFFFFF)
-        self.assertEquals(add32(0x7FFFFFFF, 1), 0x80000000)
+        self.assertEqual(add32(10, -6), 4)
+        self.assertEqual(add32(6, -10), -4&0xFFFFFFFF)
+        self.assertEqual(add32(0x80000000, -1), 0x7FFFFFFF)
+        self.assertEqual(add32(0x7FFFFFFF, 1), 0x80000000)
 
     def testChecksum(self):
         "Test calcChecksum function"
-        self.assertEquals(calcChecksum(b""), 0)
-        self.assertEquals(calcChecksum(b"\1"), 0x01000000)
-        self.assertEquals(calcChecksum(b"\x01\x02\x03\x04\x10\x20\x30\x40"), 0x11223344)
-        self.assertEquals(calcChecksum(b"\x81"), 0x81000000)
-        self.assertEquals(calcChecksum(b"\x81\x02"), 0x81020000)
-        self.assertEquals(calcChecksum(b"\x81\x02\x03"), 0x81020300)
-        self.assertEquals(calcChecksum(b"\x81\x02\x03\x04"), 0x81020304)
-        self.assertEquals(calcChecksum(b"\x81\x02\x03\x04\x05"), 0x86020304)
-        self.assertEquals(calcChecksum(b"\x41\x02\x03\x04\xD0\x20\x30\x40"), 0x11223344)
-        self.assertEquals(calcChecksum(b"\xD1\x02\x03\x04\x40\x20\x30\x40"), 0x11223344)
-        self.assertEquals(calcChecksum(b"\x81\x02\x03\x04\x90\x20\x30\x40"), 0x11223344)
-        self.assertEquals(calcChecksum(b"\x7F\xFF\xFF\xFF\x00\x00\x00\x01"), 0x80000000)
+        self.assertEqual(calcChecksum(b""), 0)
+        self.assertEqual(calcChecksum(b"\1"), 0x01000000)
+        self.assertEqual(calcChecksum(b"\x01\x02\x03\x04\x10\x20\x30\x40"), 0x11223344)
+        self.assertEqual(calcChecksum(b"\x81"), 0x81000000)
+        self.assertEqual(calcChecksum(b"\x81\x02"), 0x81020000)
+        self.assertEqual(calcChecksum(b"\x81\x02\x03"), 0x81020300)
+        self.assertEqual(calcChecksum(b"\x81\x02\x03\x04"), 0x81020304)
+        self.assertEqual(calcChecksum(b"\x81\x02\x03\x04\x05"), 0x86020304)
+        self.assertEqual(calcChecksum(b"\x41\x02\x03\x04\xD0\x20\x30\x40"), 0x11223344)
+        self.assertEqual(calcChecksum(b"\xD1\x02\x03\x04\x40\x20\x30\x40"), 0x11223344)
+        self.assertEqual(calcChecksum(b"\x81\x02\x03\x04\x90\x20\x30\x40"), 0x11223344)
+        self.assertEqual(calcChecksum(b"\x7F\xFF\xFF\xFF\x00\x00\x00\x01"), 0x80000000)
 
     def testFontFileChecksum(self):
         "Tests TTFontFile and TTF parsing code"
@@ -151,21 +203,21 @@ class TTFontFileTestCase(NearTestCase):
         subset = TTFontFile(getBytesIO(subset), 0)
         for tag in ('cmap', 'head', 'hhea', 'hmtx', 'maxp', 'name', 'OS/2',
                     'post', 'cvt ', 'fpgm', 'glyf', 'loca', 'prep'):
-            self.assert_(subset.get_table(tag))
+            self.assertTrue(subset.get_table(tag))
 
         subset.seek_table('loca')
         for n in range(4):
             pos = subset.read_ushort()    # this is actually offset / 2
-            self.failIf(pos % 2 != 0, "glyph %d at +%d should be long aligned" % (n, pos * 2))
+            self.assertFalse(pos % 2 != 0, "glyph %d at +%d should be long aligned" % (n, pos * 2))
 
-        self.assertEquals(subset.name, b"BitstreamVeraSans-Roman")
-        self.assertEquals(subset.flags, FF_SYMBOLIC)
-        self.assertEquals(subset.italicAngle, 0.0)
+        self.assertEqual(subset.name, b"BitstreamVeraSans-Roman")
+        self.assertEqual(subset.flags, FF_SYMBOLIC)
+        self.assertEqual(subset.italicAngle, 0.0)
         self.assertNear(subset.ascent,759.765625)
         self.assertNear(subset.descent,-240.234375)
-        self.assertEquals(subset.capHeight, 759.765625)
+        self.assertEqual(subset.capHeight, 759.765625)
         self.assertNear(subset.bbox, [-183.10546875, -235.83984375, 1287.109375, 928.22265625])
-        self.assertEquals(subset.stemV, 87)
+        self.assertEqual(subset.stemV, 87)
 
     def testFontMaker(self):
         "Tests TTFontMaker class"
@@ -175,8 +227,8 @@ class TTFontFileTestCase(NearTestCase):
         ttf.add("head", b"12345678xxxx")
         stm = ttf.makeStream()
         ttf = TTFontParser(getBytesIO(stm), 0)
-        self.assertEquals(ttf.get_table("ABCD"), b"xyzzy")
-        self.assertEquals(ttf.get_table("QUUX"), b"123")
+        self.assertEqual(ttf.get_table("ABCD"), b"xyzzy")
+        self.assertEqual(ttf.get_table("QUUX"), b"123")
 
 
 class TTFontFaceTestCase(unittest.TestCase):
@@ -188,18 +240,18 @@ class TTFontFaceTestCase(unittest.TestCase):
         doc = PDFDocument()
         fontDescriptor = face.addSubsetObjects(doc, "TestFont", [ 0x78, 0x2017 ])
         fontDescriptor = doc.idToObject[fontDescriptor.name].dict
-        self.assertEquals(fontDescriptor['Type'], '/FontDescriptor')
-        self.assertEquals(fontDescriptor['Ascent'], face.ascent)
-        self.assertEquals(fontDescriptor['CapHeight'], face.capHeight)
-        self.assertEquals(fontDescriptor['Descent'], face.descent)
-        self.assertEquals(fontDescriptor['Flags'], (face.flags & ~FF_NONSYMBOLIC) | FF_SYMBOLIC)
-        self.assertEquals(fontDescriptor['FontName'], "/TestFont")
-        self.assertEquals(fontDescriptor['FontBBox'].sequence, face.bbox)
-        self.assertEquals(fontDescriptor['ItalicAngle'], face.italicAngle)
-        self.assertEquals(fontDescriptor['StemV'], face.stemV)
+        self.assertEqual(fontDescriptor['Type'], '/FontDescriptor')
+        self.assertEqual(fontDescriptor['Ascent'], face.ascent)
+        self.assertEqual(fontDescriptor['CapHeight'], face.capHeight)
+        self.assertEqual(fontDescriptor['Descent'], face.descent)
+        self.assertEqual(fontDescriptor['Flags'], (face.flags & ~FF_NONSYMBOLIC) | FF_SYMBOLIC)
+        self.assertEqual(fontDescriptor['FontName'], "/TestFont")
+        self.assertEqual(fontDescriptor['FontBBox'].sequence, face.bbox)
+        self.assertEqual(fontDescriptor['ItalicAngle'], face.italicAngle)
+        self.assertEqual(fontDescriptor['StemV'], face.stemV)
         fontFile = fontDescriptor['FontFile2']
         fontFile = doc.idToObject[fontFile.name]
-        self.assert_(fontFile.content != "")
+        self.assertTrue(fontFile.content != "")
 
 
 class TTFontTestCase(NearTestCase):
@@ -208,28 +260,66 @@ class TTFontTestCase(NearTestCase):
     def testStringWidth(self):
         "Test TTFont.stringWidth"
         font = TTFont("Vera", "Vera.ttf")
-        self.assert_(font.stringWidth("test", 10) > 0)
+        self.assertTrue(font.stringWidth("test", 10) > 0)
         width = font.stringWidth(utf8(0x2260) * 2, 1000)
         expected = font.face.getCharWidth(0x2260) * 2
         self.assertNear(width,expected)
 
+    def testTTFontFromBytesIO(self):
+        "Test loading TTFont from in-memory file"
+        # Direct source: https://github.com/tmbdev/hocr-tools/blob/master/hocr-pdf
+        # Glyphless variation of vedaal's invisible font retrieved from
+        # http://www.angelfire.com/pr/pgpf/if.html, which says:
+        # 'Invisible font' is unrestricted freeware. Enjoy, Improve, Distribute freely
+        import io, zlib, base64
+        font = """
+        eJzdlk1sG0UUx/+zs3btNEmrUKpCPxikSqRS4jpfFURUagmkEQQoiRXgAl07Y3vL2mvt2ml8APXG
+        hQPiUEGEVDhWVHyIC1REPSAhBOWA+BCgSoULUqsKcWhVBKjhzfPU+VCi3Flrdn7vzZv33ryZ3TUE
+        gC6chsTx8fHck1ONd98D0jnS7jn26GPjyMIleZhk9fT0wcHFl1/9GRDPkTxTqHg1dMkzJH9CbbTk
+        xbWlJfKEdB+Np0pBswi+nH/Nvay92VtfJp4nvEztUJkUHXsdksUOkveXK/X5FNuLD838ICx4dv4N
+        I1e8+ZqbxwCNP2jyqXoV/fmhy+WW/2SqFsb1pX68SfEpZ/TCrI3aHzcP//jitodvYmvL+6Xcr5mV
+        vb1ScCzRnPRPfz+LsRSWNasuwRrZlh1sx0E8AriddyzEDfE6EkglFhJDJO5u9fJbFJ0etEMB78D5
+        4Djm/7kjT0wqhSNURyS+u/2MGJKRu+0ExNkrt1pJti9p2x6b3TBJgmUXuzgnDmI8UWMbkVxeinCw
+        Mo311/l/v3rF7+01D+OkZYE0PrbsYAu+sSyxU0jLLtIiYzmBrFiwnCT9FcsdOOK8ZHbFleSn0znP
+        nDCnxbnAnGT9JeYtrP+FOcV8nTlNnsoc3bBAD85adtCNRcsSffjBsoseca/lBE7Q09LiJOm/ttyB
+        0+IqcwfncJt5q4krO5k7jV7uY+5m7mPebuLKUea7iHvk48w72OYF5rvZT8C8k/WvMN/Dc19j3s02
+        bzPvZZv3me9j/ox5P9t/xdzPzPVJcc7yGnPL/1+GO1lPVTXM+VNWOTRRg0YRHgrUK5yj1kvaEA1E
+        xAWiCtl4qJL2ADKkG6Q3XxYjzEcR0E9hCj5KtBd1xCxp6jV5mKP7LJBr1nTRK2h1TvU2w0akCmGl
+        5lWbBzJqMJsdyaijQaCm/FK5HqspHetoTtMsn4LO0T2mlqcwmlTVOT/28wGhCVKiNANKLiJRlxqB
+        F603axQznIzRhDSq6EWZ4UUs+xud0VHsh1U1kMlmNwu9kTuFaRqpURU0VS3PVmZ0iE7gct0MG/8+
+        2fmUvKlfRLYmisd1w8pk1LSu1XUlryM1MNTH9epTftWv+16gIh1oL9abJZyjrfF5a4qccp3oFAcz
+        Wxxx4DpvlaKKxuytRDzeth5rW4W8qBFesvEX8RFRmLBHoB+TpCmRVCCb1gFCruzHqhhW6+qUF6tC
+        pL26nlWN2K+W1LhRjxlVGKmRTFYVo7CiJug09E+GJb+QocMCPMWBK1wvEOfRFF2U0klK8CppqqvG
+        pylRc2Zn+XDQWZIL8iO5KC9S+1RekOex1uOyZGR/w/Hf1lhzqVfFsxE39B/ws7Rm3N3nDrhPuMfc
+        w3R/aE28KsfY2J+RPNp+j+KaOoCey4h+Dd48b9O5G0v2K7j0AM6s+5WQ/E0wVoK+pA6/3bup7bJf
+        CMGjwvxTsr74/f/F95m3TH9x8o0/TU//N+7/D/ScVcA=
+        """.encode('latin1')
+        uncompressed = zlib.decompress(getattr(base64,'decodebytes' if isPy3 else 'decodestring')(font))
+        ttf = io.BytesIO(uncompressed)
+        setattr(ttf ,"name", "(invisible.ttf)")
+        font = TTFont('invisible', ttf)
+        for coord in font.face.bbox:
+            self.assertEqual(coord, 0)
+        pdfmetrics.registerFont(font)
+        _simple_subset_generation('test_pdfbase_ttffonts_invisible.pdf',2,fonts=('invisible',))
+
     def testSplitString(self):
         "Tests TTFont.splitString"
         doc = PDFDocument()
         font = TTFont("Vera", "Vera.ttf")
         text = b"".join(utf8(i) for i in range(511))
         allchars = b"".join(int2Byte(i) for i in range(256))
-        nospace = allchars[:32] + allchars[33:]
-        chunks = [(0, allchars), (1, nospace)]
-        self.assertEquals(font.splitString(text, doc), chunks)
+        chunks = [(0, allchars[:0xa0]+b' '+allchars[0xa0:])] + [
+                (1, allchars[1:32] + allchars[33:]) if rl_config.reserveTTFNotdef else
+                (1, allchars[:32] + allchars[33:-1])]
+        self.assertEqual(font.splitString(text, doc), chunks)
         # Do it twice
-        self.assertEquals(font.splitString(text, doc), chunks)
+        self.assertEqual(font.splitString(text, doc), chunks)
 
         text = b"".join(utf8(i) for i in range(510, -1, -1))
-        allchars = b"".join(int2Byte(i) for i in range(255, -1, -1))
-        nospace = allchars[:223] + allchars[224:]
-        chunks = [(1, nospace), (0, allchars)]
-        self.assertEquals(font.splitString(text, doc), chunks)
+        revver = (lambda b: map(int2Byte,reversed(b))) if isPy3 else (lambda b: reversed(list(b)))
+        chunks = [(i[0],b"".join(revver(i[1]))) for i in reversed(chunks)]
+        self.assertEqual(font.splitString(text, doc), chunks)
 
     def testSplitStringSpaces(self):
         # In order for justification (word spacing) to work, the space
@@ -241,9 +331,9 @@ class TTFontTestCase(NearTestCase):
         text = b"".join(utf8(i) for i in range(512, -1, -1))
         chunks = font.splitString(text, doc)
         state = font.state[doc]
-        self.assertEquals(state.assignments[32], 32)
-        self.assertEquals(state.subsets[0][32], 32)
-        self.assertEquals(state.subsets[1][32], 32)
+        self.assertEqual(state.assignments[32], 32)
+        self.assertEqual(state.subsets[0][32], 32)
+        self.assertEqual(state.subsets[1][32], 32)
 
     def testSubsetInternalName(self):
         "Tests TTFont.getSubsetInternalName"
@@ -254,10 +344,10 @@ class TTFontTestCase(NearTestCase):
         font.splitString(text, doc)
         self.assertRaises(IndexError, font.getSubsetInternalName, -1, doc)
         self.assertRaises(IndexError, font.getSubsetInternalName, 3, doc)
-        self.assertEquals(font.getSubsetInternalName(0, doc), "/F1+0")
-        self.assertEquals(font.getSubsetInternalName(1, doc), "/F1+1")
-        self.assertEquals(font.getSubsetInternalName(2, doc), "/F1+2")
-        self.assertEquals(doc.delayedFonts, [font])
+        self.assertEqual(font.getSubsetInternalName(0, doc), "/F1+0")
+        self.assertEqual(font.getSubsetInternalName(1, doc), "/F1+1")
+        self.assertEqual(font.getSubsetInternalName(2, doc), "/F1+2")
+        self.assertEqual(doc.delayedFonts, [font])
 
     def testAddObjectsEmpty(self):
         "TTFont.addObjects should not fail when no characters were used"
@@ -273,10 +363,10 @@ class TTFontTestCase(NearTestCase):
         font.splitString('a', doc)            # create some subset
         doc = PDFDocument()
         font.addObjects(doc)
-        self.assertEquals(font.frozen, 0)
-        self.assertEquals(font.nextCode, 0)
-        self.assertEquals(font.subsets, [])
-        self.assertEquals(font.assignments, {})
+        self.assertEqual(font.frozen, 0)
+        self.assertEqual(font.nextCode, 0)
+        self.assertEqual(font.subsets, [])
+        self.assertEqual(font.assignments, {})
         font.splitString('ba', doc)           # should work
 
     def testParallelConstruction(self):
@@ -287,12 +377,12 @@ class TTFontTestCase(NearTestCase):
             doc1 = PDFDocument()
             doc2 = PDFDocument()
             font = TTFont("Vera", "Vera.ttf")
-            self.assertEquals(font.splitString('hello ', doc1), [(0, b'hello ')])
-            self.assertEquals(font.splitString('hello ', doc2), [(0, b'hello ')])
-            self.assertEquals(font.splitString(u'\u0410\u0411'.encode('UTF-8'), doc1), [(0, b'\x80\x81')])
-            self.assertEquals(font.splitString(u'\u0412'.encode('UTF-8'), doc2), [(0, b'\x80')])
+            self.assertEqual(font.splitString('hello ', doc1), [(0, b'hello ')])
+            self.assertEqual(font.splitString('hello ', doc2), [(0, b'hello ')])
+            self.assertEqual(font.splitString(u'\u0410\u0411'.encode('UTF-8'), doc1), [(0, b'\x80\x81')])
+            self.assertEqual(font.splitString(u'\u0412'.encode('UTF-8'), doc2), [(0, b'\x80')])
             font.addObjects(doc1)
-            self.assertEquals(font.splitString(u'\u0413'.encode('UTF-8'), doc2), [(0, b'\x81')])
+            self.assertEqual(font.splitString(u'\u0413'.encode('UTF-8'), doc2), [(0, b'\x81')])
             font.addObjects(doc2)
         finally:
             rl_config.ttfAsciiReadable = ttfAsciiReadable
@@ -309,22 +399,22 @@ class TTFontTestCase(NearTestCase):
             internalName = font.getSubsetInternalName(0, doc)[1:]
             font.addObjects(doc)
             pdfFont = doc.idToObject[internalName]
-            self.assertEquals(doc.idToObject['BasicFonts'].dict[internalName], pdfFont)
-            self.assertEquals(pdfFont.Name, internalName)
-            self.assertEquals(pdfFont.BaseFont, "AAAAAA+BitstreamVeraSans-Roman")
-            self.assertEquals(pdfFont.FirstChar, 0)
-            self.assertEquals(pdfFont.LastChar, 127)
-            self.assertEquals(len(pdfFont.Widths.sequence), 128)
+            self.assertEqual(doc.idToObject['BasicFonts'].dict[internalName], pdfFont)
+            self.assertEqual(pdfFont.Name, internalName)
+            self.assertEqual(pdfFont.BaseFont, "AAAAAA+BitstreamVeraSans-Roman")
+            self.assertEqual(pdfFont.FirstChar, 0)
+            self.assertEqual(pdfFont.LastChar, 127)
+            self.assertEqual(len(pdfFont.Widths.sequence), 128)
             toUnicode = doc.idToObject[pdfFont.ToUnicode.name]
-            self.assert_(toUnicode.content != "")
+            self.assertTrue(toUnicode.content != "")
             fontDescriptor = doc.idToObject[pdfFont.FontDescriptor.name]
-            self.assertEquals(fontDescriptor.dict['Type'], '/FontDescriptor')
+            self.assertEqual(fontDescriptor.dict['Type'], '/FontDescriptor')
         finally:
             rl_config.ttfAsciiReadable = ttfAsciiReadable
 
     def testMakeToUnicodeCMap(self):
         "Test makeToUnicodeCMap"
-        self.assertEquals(makeToUnicodeCMap("TestFont", [ 0x1234, 0x4321, 0x4242 ]),
+        self.assertEqual(makeToUnicodeCMap("TestFont", [ 0x1234, 0x4321, 0x4242 ]),
 """/CIDInit /ProcSet findresource begin
 12 dict begin
 begincmap
