@@ -12,6 +12,7 @@ from reportlab.pdfgen import canvas   # gmcm 2000/10/13, pdfgen now a package
 from reportlab.lib.units import inch, cm
 from reportlab.lib import colors
 from reportlab.lib.utils import haveImages, fileName2FSEnc
+from reportlab.lib.boxstuff import rectCorner
 
 #################################################################
 #
@@ -204,7 +205,7 @@ def makeDocument(filename, pageCallBack=None):
 
     #quickie encoding test: when canvas encoding not set,
     #the following should do (tm), (r) and (c)
-    msg_uni = 'copyright\u00A9 trademark\u2122 registered\u00AE scissors\u2702: ReportLab in unicode!'
+    msg_uni = u'copyright\u00A9 trademark\u2122 registered\u00AE scissors\u2702: ReportLab in unicode!'
     msg_utf8 = msg_uni.replace('unicode','utf8').encode('utf8')
     c.drawString(100, 100, msg_uni)
     c.drawString(100, 80, msg_utf8)
@@ -845,31 +846,63 @@ cost to performance.""")
         c.drawString(inch, 5.75*inch,
         'anchored by respective corners - use both a wide and a tall one as tests')
         x = 0.25 * inch
-        for anchor in ['nw','n','ne','w','c','e','sw','s','se']:
+        y = 5*inch
+        for anchor in ('nw','n','ne','w','c','e','sw','s','se'):
             x += 0.75*inch
-            c.rect(x, 5*inch, 0.6*inch, 0.6*inch)
+            c.rect(x, y, 0.6*inch, 0.6*inch)
             c.drawImage(
-                    gif, x, 5*inch,
+                    gif, x, y,
                     width=0.6*inch, height=0.6*inch,
                     preserveAspectRatio=True,
                     anchor=anchor
                     )
-            c.drawString(x, 4.9*inch, anchor)
+            c.drawString(x, y-0.1*inch, anchor)
 
         x = 0.25 * inch
+        y = 4*inch
         tall_red = os.path.join(testsFolder,'tall_red.png')
-        for anchor in ['nw','n','ne','w','c','e','sw','s','se']:
+        for anchor in ('nw','n','ne','w','c','e','sw','s','se'):
             x += 0.75*inch
-            c.rect(x, 4*inch, 0.6*inch, 0.6*inch)
+            c.rect(x, y, 0.6*inch, 0.6*inch)
             c.drawImage(
-                    tall_red, x, 4*inch,
+                    tall_red, x, y,
                     width=0.6*inch, height=0.6*inch,
                     preserveAspectRatio=True,
                     anchor=anchor
                     )
-            c.drawString(x, 3.9*inch, anchor)
+            c.drawString(x, y-0.1*inch, anchor)
 
+        #Andy's positioning code fails for this case when the image is not reaching the limit
+        x = 0.25 * inch
+        y = 3*inch
+        tall_red = os.path.join(testsFolder,'tall_red.png')
+        for anchor in ('nw','n','ne','w','c','e','sw','s','se'):
+            x += 0.75*inch
+            c.rect(x, 3*inch, 0.7*inch, 0.7*inch)
+            c.drawImage(
+                    tall_red, x, 3*inch,
+                    width=0.6*inch, height=0.6*inch,
+                    preserveAspectRatio=True,
+                    anchor=anchor
+                    )
+            c.drawString(x, y-0.1*inch, anchor)
 
+        #fix by using anchorAtXY
+        x = 0.25 * inch
+        y = 2*inch
+        tall_red = os.path.join(testsFolder,'tall_red.png')
+        for anchor in ('nw','n','ne','w','c','e','sw','s','se'):
+            x += 0.75*inch
+            c.rect(x, y, 0.7*inch, 0.7*inch)
+            ax, ay = rectCorner(x, y, 0.7*inch, 0.7*inch, anchor)
+            c.drawImage(
+                    tall_red, ax, ay,
+                    width=0.6*inch, height=0.6*inch,
+                    preserveAspectRatio=True,
+                    anchor=anchor,
+                    anchorAtXY=True,
+                    )
+            c.drawString(x, y-0.1*inch, anchor)
 
         c.showPage()
 
@@ -1119,6 +1152,7 @@ class PdfgenTestCase(unittest.TestCase):
 
     def test5(self,uopw=None):
         from reportlab.lib.pagesizes import A4,LETTER
+        from reportlab.lib.rl_accel import fp_str
         if uopw:
             from reportlab.lib import pdfencrypt
             encrypt = pdfencrypt.StandardEncryption(uopw[0], uopw[1])
@@ -1128,31 +1162,33 @@ class PdfgenTestCase(unittest.TestCase):
             canv._doc.encrypt = encrypt
         else:
             canv = canvas.Canvas(outputfile('test_pdfgen_general_page_sizes.pdf'),pagesize=A4)
+        def tstr(*args):
+            return tuple((fp_str(a) for a in args))
         canv.setFont('Helvetica',10)
         S = A4
-        canv.drawString(0,S[1]-10,'Top Left=(%s,%s) Page Size=%s x %s' % (0,S[1],S[0],S[1]))
-        canv.drawCentredString(0.5*S[0],0.5*S[1],'Center =(%s,%s) Page Size=%s x %s' % (0.5*S[0],0.5*S[1],S[0],S[1]))
-        canv.drawRightString(S[0],2,'Bottom Right=(%s,%s) Page Size=%s x %s' % (S[0],0,S[0],S[1]))
+        canv.drawString(0,S[1]-10,'Top Left=(%s,%s) Page Size=%s x %s' % tstr(0,S[1],S[0],S[1]))
+        canv.drawCentredString(0.5*S[0],0.5*S[1],'Center =(%s,%s) Page Size=%s x %s' % tstr(0.5*S[0],0.5*S[1],S[0],S[1]))
+        canv.drawRightString(S[0],2,'Bottom Right=(%s,%s) Page Size=%s x %s' % tstr(S[0],0,S[0],S[1]))
         canv.showPage()
         S = LETTER
         canv.setPageSize(S)
-        canv.drawString(0,S[1]-10,'Top Left=(%s,%s) Page Size=%s x %s' % (0,S[1],S[0],S[1]))
-        canv.drawCentredString(0.5*S[0],0.5*S[1],'Center =(%s,%s) Page Size=%s x %s' % (0.5*S[0],0.5*S[1],S[0],S[1]))
-        canv.drawRightString(S[0],2,'Bottom Right=(%s,%s) Page Size=%s x %s' % (S[0],0,S[0],S[1]))
+        canv.drawString(0,S[1]-10,'Top Left=(%s,%s) Page Size=%s x %s' % tstr(0,S[1],S[0],S[1]))
+        canv.drawCentredString(0.5*S[0],0.5*S[1],'Center =(%s,%s) Page Size=%s x %s' % tstr(0.5*S[0],0.5*S[1],S[0],S[1]))
+        canv.drawRightString(S[0],2,'Bottom Right=(%s,%s) Page Size=%s x %s' % tstr(S[0],0,S[0],S[1]))
         canv.showPage()
         S = A4
         canv.setPageSize(S)
         canv.setPageRotation(180)
-        canv.drawString(0,S[1]-10,'Top Left=(%s,%s) Page Size=%s x %s' % (0,S[1],S[0],S[1]))
-        canv.drawCentredString(0.5*S[0],0.5*S[1],'Center =(%s,%s) Page Size=%s x %s' % (0.5*S[0],0.5*S[1],S[0],S[1]))
-        canv.drawRightString(S[0],2,'Bottom Right=(%s,%s) Page Size=%s x %s' % (S[0],0,S[0],S[1]))
+        canv.drawString(0,S[1]-10,'Top Left=(%s,%s) Page Size=%s x %s' % tstr(0,S[1],S[0],S[1]))
+        canv.drawCentredString(0.5*S[0],0.5*S[1],'Center =(%s,%s) Page Size=%s x %s' % tstr(0.5*S[0],0.5*S[1],S[0],S[1]))
+        canv.drawRightString(S[0],2,'Bottom Right=(%s,%s) Page Size=%s x %s' % tstr(S[0],0,S[0],S[1]))
         canv.showPage()
         S = A4[1],A4[0]
         canv.setPageSize(S)
         canv.setPageRotation(0)
-        canv.drawString(0,S[1]-30,'Top Left=(%s,%s) Page Size=%s x %s' % (0,S[1],S[0],S[1]))
-        canv.drawCentredString(0.5*S[0],0.5*S[1],'Center =(%s,%s) Page Size=%s x %s' % (0.5*S[0],0.5*S[1],S[0],S[1]))
-        canv.drawRightString(S[0],32,'Bottom Right=(%s,%s) Page Size=%s x %s' % (S[0],0,S[0],S[1]))
+        canv.drawString(0,S[1]-30,'Top Left=(%s,%s) Page Size=%s x %s' % tstr(0,S[1],S[0],S[1]))
+        canv.drawCentredString(0.5*S[0],0.5*S[1],'Center =(%s,%s) Page Size=%s x %s' % tstr(0.5*S[0],0.5*S[1],S[0],S[1]))
+        canv.drawRightString(S[0],32,'Bottom Right=(%s,%s) Page Size=%s x %s' % tstr(S[0],0,S[0],S[1]))
         canv.showPage()
         canv.save()
 
@@ -1180,6 +1216,15 @@ class PdfgenTestCase(unittest.TestCase):
         c.showPage()
         c.save()
 
+    def testOSFile(self):
+        import io
+        fd = os.open(outputfile('test_pdfgen_osfile.pdf'),getattr(os,'O_BINARY',0)|os.O_WRONLY|os.O_CREAT)
+        fn = io.FileIO(fd,'wb')
+        c = canvas.Canvas(fn)
+        c.drawString(100, 700, 'Hello World my filedescriptor is %d!' % fd)
+        c.showPage()
+        c.save()
+
 def trySomeColors(C,enforceColorSpace=None):
     from reportlab.lib.utils import getBytesIO
     out=getBytesIO()
