@@ -1,12 +1,16 @@
 from random import randint
+from xml.sax.saxutils import escape as xmlEscape
+from reportlab import xrange
 from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation
 setOutDir(__name__)
 import os,unittest
-from reportlab.platypus import Spacer, SimpleDocTemplate, Table, TableStyle, ListFlowable, ListItem, Paragraph, PageBreak
+from reportlab.platypus import Spacer, SimpleDocTemplate, Table, TableStyle, ListFlowable, ListItem, \
+        Paragraph, PageBreak, DDIndenter, MultiCol
 from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
 from reportlab.lib.units import inch, cm
 from reportlab.lib.utils import simpleSplit
 from reportlab.lib import colors
+from reportlab.lib import randomtext
 
 TEXTS=[
 '''We have already seen that the notion of level of grammaticalness is,
@@ -57,15 +61,16 @@ class ListsTestCase(unittest.TestCase):
 
     def test1(self):
         styleSheet = getSampleStyleSheet()
-        doc = SimpleDocTemplate(outputfile('test_platypus_lists1.pdf'))
+        doc = SimpleDocTemplate(outputfile('test_platypus_lists1.pdf'),showBoundary=True)
         story=[]
         sty = [ ('GRID',(0,0),(-1,-1),1,colors.green),
             ('BOX',(0,0),(-1,-1),2,colors.red),
             ]
         normal = styleSheet['BodyText']
+        bold = normal.clone('bold',fontName='Helvetica-Bold')
         lpSty = normal.clone('lpSty',spaceAfter=18)
-        data = [[str(i+1), Paragraph("xx "* (i%10), styleSheet["BodyText"]), Paragraph(("blah "*(i%40)), normal)] for i in range(5)]
-        data1 = [[str(i+1), Paragraph(["zz ","yy "][i]*(i+3), styleSheet["BodyText"]), Paragraph(("duh  "*(i+3)), normal)] for i in range(2)]
+        data = [[str(i+1), Paragraph("xx "* (i%10), styleSheet["BodyText"]), Paragraph(("blah "*(i%40)), normal)] for i in xrange(5)]
+        data1 = [[str(i+1), Paragraph(["zz ","yy "][i]*(i+3), styleSheet["BodyText"]), Paragraph(("duh  "*(i+3)), normal)] for i in xrange(2)]
         OL = ListFlowable(
             [
             Paragraph("A table with 5 rows", lpSty),
@@ -157,7 +162,45 @@ http://www.biometricassociates.com/downloads/user-guides/make-the-url-even-longe
                     ),
             )
         
-        
+        story.append(PageBreak())
+        story.append(Paragraph("DDIndenter", style=normal))
+        story.append(Paragraph("Coffee",style=bold))
+        story.append(DDIndenter(Paragraph("Black hot drink",style=normal),leftIndent=36))
+        story.append(Paragraph("Milk",style=bold))
+        story.append(DDIndenter(Paragraph("White cold drink",style=normal),leftIndent=36))
+        story.append(Paragraph("Whiskey",style=bold))
+        story.append(DDIndenter(Paragraph("A nice alcoholic drink",style=normal),leftIndent=36))
+        story.append(PageBreak())
+        story.append(Paragraph("MultiCol", style=normal))
+        RT = 'STARTUP COMPUTERS BLAH BUZZWORD STARTREK PRINTING PYTHON CHOMSKY CHOMSKY'.split()
+        for i in xrange(5):
+            topic = RT[randint(0,len(RT)-1)]
+            np = randint(2,6)
+            story.append(
+                    MultiCol([
+                        [Paragraph('Column %d' % (i+1,),style=bold)],
+                        [],
+                        [Paragraph(xmlEscape(randomtext.randomText(topic,randint(1,7))),style=normal) for j in xrange(np)]
+                        ],
+                        widths=['20%',3,'80%'],
+                        )
+                    )
+
+        story.append(PageBreak())
+        story.append(Paragraph("MultiCol 2", style=normal))
+        for i in xrange(5):
+            topic = RT[randint(0,len(RT)-1)]
+            np = randint(2,6)
+            story.append(
+                    MultiCol([
+                        ([Paragraph('Column %d' % (i+1,),style=bold)]+
+                        [Paragraph(xmlEscape(randomtext.randomText(topic,randint(1,7))),style=normal) for j in xrange(np)]),
+                        [],
+                        [Paragraph(xmlEscape(randomtext.randomText(topic,randint(1,7))),style=normal) for j in xrange(np)]
+                        ],
+                        widths=['50%',5,'50%'],
+                        )
+                    )
         
         doc.build(story)
 
