@@ -7,6 +7,7 @@ from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, p
 setOutDir(__name__)
 import sys, os, unittest
 from operator import truth
+from reportlab.pdfgen.canvas import Canvas
 from reportlab.pdfbase.pdfmetrics import stringWidth, registerFont, registerFontFamily
 from reportlab.pdfbase.ttfonts import TTFont
 from reportlab.platypus.paraparser import ParaParser
@@ -108,7 +109,6 @@ class ParagraphCorners(unittest.TestCase):
 
     def test3(self):
         '''compare CJK splitting in some edge cases'''
-        from reportlab.pdfgen.canvas import Canvas
         from reportlab.platypus.paragraph import Paragraph
         from reportlab.lib.styles import ParagraphStyle
         from reportlab.pdfbase import pdfmetrics
@@ -499,6 +499,54 @@ component delimits the traditional practice of grammarians.'''
         doc = MyDocTemplate(outputfile('test_platypus_imageandflowables_rtl.pdf'),showBoundary=1)
         doc.multiBuild(story)
 
+    def test_splitJustBug(self):
+        """test that justified paragraphs with </br>last line split properly
+        bug reported by Niharika Singh <nsingh@shoobx.com>
+        """
+        measures = []
+        def _odW(canv,name,label):
+            measures.append((label,canv._curr_tx_info['cur_x']))
+        text = '''<para><onDraw name="_odW" label="start"/>First line<onDraw name="_odW" label="end"/><br/><onDraw name="_odW" label="start"/>Second line<onDraw name="_odW" label="end"/><br/><onDraw name="_odW" label="start"/>split here<onDraw name="_odW" label="end"/><br/><onDraw name="_odW" label="start"/>Third line should not be justified<onDraw name="_odW" label="end"/><br/></para>'''
+        normal = getSampleStyleSheet()['BodyText']
+        normal.fontName = "Helvetica"
+        normal.fontSize = 10
+        normal.leading = 12
+        normal.alignment = TA_JUSTIFY
+        canv = Canvas(outputfile('test_splitJustBug.pdf'))
+        canv._odW = _odW
+        W, H = canv._pagesize
+        aW = W-2*72
+        aH = H-2*72
+        x = 72
+        y = H-72
+        P = Paragraph(text,normal)
+        w,h = P.wrap(aW,aH)
+        P.drawOn(canv,x,y)
+        M0 = measures[:]
+        measures[:] = []
+        y -= h
+        aH -= h
+        P = Paragraph(text,normal)
+        w,h = P.wrap(W-2*72,H-2*72)
+        P1,P2 = P.split(aW,37)
+        w,h = P1.wrap(aW,37)
+        P1.drawOn(canv,x,y)
+        y -= h
+        aH -= h
+        w,h = P2.wrap(aW,aH)
+        P2.drawOn(canv,x,y)
+        y -= h
+        aH -= h
+        canv.save()
+        self.assertEqual(M0,measures,"difference detected in justified split Paragraph rendering")
+
+    def test_unicharCodeSafety(self):
+        """test a bug reported by ravi prakash giri <raviprakashgiri@gmail.com>"""
+        normal = getSampleStyleSheet()['BodyText']
+        self.assertRaises(Exception,Paragraph,
+                """<unichar code="open('/tmp/test.txt','w').write('Hello from unichar')"/>""",
+                normal)
+
 class TwoFrameDocTemplate(BaseDocTemplate):
     "Define a simple document with two frames per page."
     
@@ -639,7 +687,6 @@ class FragmentTestCase(unittest.TestCase):
         bt.fontSize = 10
         bt.leading = 12
         bt.alignment = TA_JUSTIFY
-        from reportlab.pdfgen.canvas import Canvas
         canv = Canvas(outputfile('test_platypus_paragraphs_hyphenations.pdf'))
         x = 72
         y = canv._pagesize[1] - 72
@@ -701,6 +748,42 @@ class FragmentTestCase(unittest.TestCase):
             y = _t(S[1],x,y,naW,aH=naH)
         canv.save()
 
+    @unittest.skipUnless(pyphen,'s')
+    def test7(self):
+        """test various ways to adjust the hypenationMinWordLength"""
+        registerFont(TTFont("Vera", "Vera.ttf"))
+        aW = 51.0236220472
+        text  = u'\u0440\u044b\u0431\u0430 \u043f\u0438\u043b\u0430 \u0438 \u0431\u0430\u0431\u0443 \u0434\u0430\u0436\u0435 \u0432\u0435\u043b\u0430 \u043d\u0430 \u043c\u0430\u044f\u043a'
+        text1 = u'\u0440\u044b\u0431\u0430 <span color="blue">\u043f\u0438\u043b\u0430</span> \u0438 \u0431\u0430\u0431\u0443 \u0434\u0430\u0436\u0435 \u0432\u0435\u043b\u0430 \u043d\u0430 \u043c\u0430\u044f\u043a'
+        tmpls = [
+                u'%(text)s',
+                u'<para hyphenationMinWordLength="%(hymwl)d">%(text)s</para>',
+                u'<para>%(text)s</para>',
+                u'%(text1)s',
+                u'<para hyphenationMinWordLength="%(hymwl)d">%(text1)s</para>',
+                u'<para>%(text1)s</para>',
+                ]
+        for ex,x,hymwl in [
+                (72,0,None),    #default is 5
+                (72,0,5),
+                (60,0,4),
+                (72,2,None),    #default is 5
+                (72,1,5),
+                (60,1,4),
+                (72,3,None),    #default is 5
+                (72,4,5),
+                (60,5,4),
+                ]:
+            kwds = dict(hyphenationMinWordLength=hymwl) if hymwl!=None else {}
+            template = tmpls[x]
+            t = template % locals()
+            p = Paragraph(
+                    template % locals(),
+                    style = ParagraphStyle('P10H5', fontSize=10, fontName="Vera", hyphenationLang="ru_RU",**kwds),
+                    )
+            w,h = p.wrap(aW,0x7fffffff)
+            self.assertEqual(h,ex,'Russion hyphenation test failed for ex=%(ex)s template=%(template)r hymwl=%(hymwl)r h=%(h)s\ntext=%(t)r' % locals())
+
 class ULTestCase(unittest.TestCase):
     "Test underlining and overstriking of paragraphs."
     def testUl(self):
@@ -794,6 +877,13 @@ phonemic and morphological analysis.''']
         a(Paragraph('''<para spaceBefore="10">This is <strike width="0.5" offset="0.125*F" color="red" kind="triple" gap="0.5">triply overstruck in red</strike></para>''',style=normal))
         a(Paragraph('''<para autoLeading="max" spaceAfter="10" spaceBefore="30">this is in 12 <font size="30">this is in 30</font> <u offset="-0.5" width="0.5" color="red"><u offset="-1.5" width="0.5" color="blue">and</u></u> <link underline="1" ucolor="blue" href="http://google.com/">the link box<sup><a color="red" ucolor="green" underline="1" href="https://www.reportlab.com">2</a></sup> is right (twice).</link></para>''',style=normal))
         a(Paragraph("",style=normal))
+        # 3.5.x had a bug with leftIndent and underlines, check that
+        left_indent = ParagraphStyle(name='left_indent',leftIndent=80,parent=styleSheet['Normal'])
+        a(Paragraph("<u>Replicating a left indent underline bug.</u>",style=left_indent))
+
+        #testing PR 49
+        a(Paragraph("This paragraph contains <strike>really</strike> large <u>spaces</u> <strike color='blue' offset='1' width='0.5'><u color='red' offset='-1' width='0.5'>between words</u></strike>. ThisLongWordDoesntFitIntoPreviousLineCausingWordsSpread",style=normal_just))
+
         doc = MyDocTemplate(outputfile('test_platypus_paragraphs_ul.pdf'))
         doc.build(story)
 
